/**************************************************************************/
/* File: unican_vme.c - Unicontrols VME board additional support          */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2009 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2002-2009 Pavel Pisa <pisa@cmp.felk.cvut.cz>             */
/* Funded by OCERA and FRESCOR IST projects                               */
/* Based on CAN driver code by Arnaud Westenberg <arnaud@wanadoo.nl>      */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/
/* This file is included in unican.c if CAN_ENABLE_VME_SUPPORT is
 * set. */


/**
 * unican_vme_init_hw_data - Initialize hardware cards
 * @candev: Pointer to candevice/board structure
 *
 * Return Value: The function always returns zero
 * File: src/unican.c
 */
int unican_vme_reset(struct candevice_t *candev)
{
	int ret;
	struct canchip_t *chip = candev->chip[0];

	ret = unican_reset(candev);

	/* Setup VME interrupt vector */
	if (ret == 0) 
		unican_writew(chip->chip_irq, chip->chip_base_addr+CL2_VME_INT_VECTOR);

	return ret;
}

/**
 * unican_vme_init_chip_data - Initialize chips
 * @candev: Pointer to candevice/board structure
 * @chipnr: Number of the CAN chip on the hardware card
 *
 * Return Value: The function always returns zero
 * File: src/unican.c
 */
int unican_vme_init_chip_data(struct candevice_t *candev, int chipnr)
{
	struct canchip_t *chip = candev->chip[chipnr];

	unican_init_chip_data(candev, chipnr);

	chip->flags |= CHIP_IRQ_VME;
	/*chip->chipspecops->irq_handler=unican_irq_handler;*/
	return 0;
}


int unican_vme_init_hw_data(struct candevice_t *candev) 
{
	unican_init_hw_data(candev);
	candev->flags |= CANDEV_PROGRAMMABLE_IRQ;

	return 0;
}

int unican_vme_register(struct hwspecops_t *hwspecops)
{
	unican_register(hwspecops);

	hwspecops->init_hw_data = unican_vme_init_hw_data;
	hwspecops->init_chip_data = unican_vme_init_chip_data;
	hwspecops->request_io = unican_request_io;
	hwspecops->reset = unican_vme_reset;
	hwspecops->release_io = unican_release_io;
	return 0;
}
